@@ -181,7 +181,7 @@ type (
 	// A BasicLit node represents a literal of basic type.
 	BasicLit struct {
 		ValuePos token.Pos   // literal position
-		Kind     token.Token // token.INT, token.FLOAT, token.IMAG, token.CHAR, or token.STRING
+		Kind     token.Token // token.INT, token.FLOAT, token.IMAG, token.CHAR, token.STRING or token.CSTRING
 		Value    string      // literal string; e.g. 42, 0x7f, 3.14, 1e-9, 2.4i, 'a', '\x7f', "foo" or `\m\n\o`
 	}
 
@@ -1114,16 +1114,10 @@ type File struct {
 	Code         []byte
 	NoEntrypoint bool // no entrypoint func to indicate the module entry point.
 	NoPkgDecl    bool // no `package xxx` declaration
-	FileType     FileType
+	IsClass      bool // is a classfile
+	IsProj       bool // is a project classfile
 }
 
-const (
-	FileTypeGo  = -1 // .go (should be negative)
-	FileTypeGop = 0  // .gop
-	FileTypeSpx = 1  // .spx
-	FileTypeGmx = 2  // .gmx
-)
-
 // Pos returns position of first character belonging to the node.
 func (f *File) Pos() token.Pos { return f.Package }
 
@@ -1139,10 +1133,11 @@ func (f *File) End() token.Pos {
 // collectively building a Go package.
 //
 type Package struct {
-	Name    string             // package name
-	Scope   *Scope             // package scope across all files
-	Imports map[string]*Object // map of package id -> package object
-	Files   map[string]*File   // Go source files by filename
+	Name    string               // package name
+	Scope   *Scope               // package scope across all files
+	Imports map[string]*Object   // map of package id -> package object
+	Files   map[string]*File     // Go+ source files by filename
+	GoFiles map[string]*ast.File // Go source files by filename
 }
 
 // Pos returns position of first character belonging to the node.
