@@ -18,7 +18,6 @@
 package cl
 
 import (
-	"context"
 	"errors"
 	"fmt"
 	"go/types"
@@ -29,8 +28,11 @@ import (
 	"strings"
 
 	"github.com/goplus/gop/ast"
+	"github.com/goplus/gop/ast/fromgo"
 	"github.com/goplus/gop/token"
 	"github.com/goplus/gox"
+	"github.com/goplus/gox/cpackages"
+	"github.com/goplus/mod/modfile"
 )
 
 const (
@@ -71,82 +73,44 @@ func (p *Errors) Error() string {
 
 // -----------------------------------------------------------------------------
 
+type Class = modfile.Classfile
+
 // Config of loading Go+ packages.
 type Config struct {
-	// Context specifies the context for the load operation.
-	// If the context is cancelled, the loader may stop early
-	// and return an ErrCancelled error.
-	// If Context is nil, the load cannot be cancelled.
-	Context context.Context
-
-	// Logf is the logger for the config.
-	// If the user provides a logger, debug logging is enabled.
-	// If the GOPACKAGESDEBUG environment variable is set to true,
-	// but the logger is nil, default to log.Printf.
-	Logf func(format string, args ...interface{})
-
-	// Dir is the directory in which to run the build system's query tool
-	// that provides information about the packages.
-	// If Dir is empty, the tool is run in the current directory.
-	Dir string
-
-	// WorkingDir is the directory in which to run gop compiler.
-	// TargetDir is the directory in which to generate Go files.
-	// If WorkingDir or TargetDir is empty, it is same as Dir.
-	WorkingDir, TargetDir string
-
-	// ModRootDir specifies root dir of this module.
-	// If ModRootDir is empty, will lookup gop.mod in all ancestor directories of Dir.
-	ModRootDir string
-
-	// CacheFile specifies where cache data to write.
-	CacheFile string
-
-	// Env is the environment to use when invoking the build system's query tool.
-	// If Env is nil, the current environment is used.
-	// As in os/exec's Cmd, only the last value in the slice for
-	// each environment key is used. To specify the setting of only
-	// a few variables, append to the current environment, as in:
-	//
-	//	opt.Env = append(os.Environ(), "GOOS=plan9", "GOARCH=386")
-	//
-	Env []string
-
-	// BuildFlags is a list of command-line flags to be passed through to
-	// the build system's query tool.
-	BuildFlags []string
-
 	// Fset provides source position information for syntax trees and types.
 	// If Fset is nil, Load will use a new fileset, but preserve Fset's value.
 	Fset *token.FileSet
 
-	// GenGoPkg is called to convert a Go+ package into Go.
-	GenGoPkg func(pkgDir string, base *Config) error
+	// WorkingDir is the directory in which to run gop compiler.
+	WorkingDir string
 
-	// PkgsLoader is the Go+ packages loader (will be set if it is nil).
-	PkgsLoader *PkgsLoader
+	// TargetDir is the directory in which to generate Go files.
+	TargetDir string
 
-	// CacheLoadPkgs = true means to cache all loaded packages.
-	CacheLoadPkgs bool
+	// C2goBase specifies base of standard c2go packages.
+	// Default is github.com/goplus/.
+	C2goBase string
 
-	// PersistLoadPkgs = true means to cache all loaded packages to disk.
-	PersistLoadPkgs bool
+	// LookupPub lookups the c2go package pubfile (named c2go.a.pub).
+	LookupPub func(pkgPath string) (pubfile string, err error)
+
+	// LookupClass lookups a class by specified file extension.
+	LookupClass func(ext string) (c *Class, ok bool)
+
+	// An Importer resolves import paths to Packages.
+	Importer types.Importer
 
 	// NoFileLine = true means not to generate file line comments.
 	NoFileLine bool
 
 	// RelativePath = true means to generate file line comments with relative file path.
 	RelativePath bool
-}
 
-func (conf *Config) Ensure() *Config {
-	if conf == nil {
-		conf = &Config{Fset: token.NewFileSet()}
-	}
-	if conf.PkgsLoader == nil {
-		initPkgsLoader(conf)
-	}
-	return conf
+	// NoAutoGenMain = true means not to auto generate main func is no entry.
+	NoAutoGenMain bool
+
+	// NoSkipConstant = true means disable optimization of skip constants
+	NoSkipConstant bool
 }
 
 type nodeInterp struct {
@@ -196,9 +160,11 @@ type baseLoader struct {
 	start token.Pos
 }
 
-func initLoader(ctx *pkgCtx, syms map[string]loader, start token.Pos, name string, fn func()) {
+func initLoader(ctx *pkgCtx, syms map[string]loader, start token.Pos, name string, fn func(), genCode bool) {
 	if name == "_" {
-		ctx.inits = append(ctx.inits, fn)
+		if genCode {
+			ctx.inits = append(ctx.inits, fn)
+		}
 		return
 	}
 	if old, ok := syms[name]; ok {
@@ -285,6 +251,7 @@ func doInitMethods(ld *typeLoader) {
 type pkgCtx struct {
 	*nodeInterp
 	*gmxSettings
+	cpkgs *cpackages.Importer
 	syms  map[string]loader
 	inits []func()
 	tylds []*typeLoader
@@ -298,11 +265,13 @@ type blockCtx struct {
 	fset         *token.FileSet
 	imports      map[string]*gox.PkgRef
 	lookups      []*gox.PkgRef
+	clookups     []*cpackages.PkgRef
+	c2goBase     string // default is `github.com/goplus/`
 	targetDir    string
-	classRecv    *ast.FieldList // avaliable when gmxSettings != nil
+	classRecv    *ast.FieldList // available when gmxSettings != nil
 	fileLine     bool
 	relativePath bool
-	fileType     int16
+	isClass      bool
 }
 
 func (bc *blockCtx) findImport(name string) (pr *gox.PkgRef, ok bool) {
@@ -312,7 +281,7 @@ func (bc *blockCtx) findImport(name string) (pr *gox.PkgRef, ok bool) {
 	for k, v := range bc.imports {
 		v.EnsureImported()
 		if v.Types != nil {
-			if v.Types.Name() != k {
+			if v.Types.Name() != k { // if we guess pkgName failed
 				delete(bc.imports, k)
 				bc.imports[v.Types.Name()] = v
 			}
@@ -398,57 +367,76 @@ func (p *pkgCtx) handleRecover(e interface{}) {
 	p.handleErr(err)
 }
 
+const (
+	defaultGoFile  = ""
+	skippingGoFile = "_skip"
+	testingGoFile  = "_test"
+)
+
 // NewPackage creates a Go+ package instance.
 func NewPackage(pkgPath string, pkg *ast.Package, conf *Config) (p *gox.Package, err error) {
-	conf = conf.Ensure()
-	dir := conf.Dir
-	if dir == "" {
-		dir, _ = os.Getwd()
-	}
 	workingDir := conf.WorkingDir
 	if workingDir == "" {
 		workingDir, _ = os.Getwd()
 	}
 	targetDir := conf.TargetDir
 	if targetDir == "" {
-		targetDir = dir
+		targetDir = workingDir
+	}
+	fset := conf.Fset
+	files := pkg.Files
+	interp := &nodeInterp{
+		fset: fset, files: files, workingDir: workingDir,
+	}
+	ctx := &pkgCtx{
+		syms: make(map[string]loader), nodeInterp: interp,
 	}
-	interp := &nodeInterp{fset: conf.Fset, files: pkg.Files, workingDir: workingDir}
-	ctx := &pkgCtx{syms: make(map[string]loader), nodeInterp: interp}
 	confGox := &gox.Config{
-		Context:         conf.Context,
-		Logf:            conf.Logf,
-		Dir:             dir,
-		ModRootDir:      conf.ModRootDir,
-		Env:             conf.Env,
-		BuildFlags:      conf.BuildFlags,
-		Fset:            conf.Fset,
-		LoadPkgs:        conf.PkgsLoader.LoadPkgs,
+		Fset:            fset,
+		Importer:        conf.Importer,
 		LoadNamed:       ctx.loadNamed,
 		HandleErr:       ctx.handleErr,
 		NodeInterpreter: interp,
-		ParseFile:       nil, // TODO
 		NewBuiltin:      newBuiltinDefault,
+		DefaultGoFile:   defaultGoFile,
+		NoSkipConstant:  conf.NoSkipConstant,
 	}
 	p = gox.NewPackage(pkgPath, pkg.Name, confGox)
-	for file, gmx := range pkg.Files {
-		if gmx.FileType == ast.FileTypeGmx {
-			ctx.gmxSettings = newGmx(p, file)
+	ctx.cpkgs = cpackages.NewImporter(&cpackages.Config{
+		Pkg: p, LookupPub: conf.LookupPub,
+	})
+	for file, gmx := range files {
+		if gmx.IsProj {
+			ctx.gmxSettings = newGmx(p, file, conf)
 			break
 		}
 	}
-	for fpath, f := range pkg.Files {
-		preloadFile(p, ctx, fpath, f, targetDir, conf)
+	for fpath, f := range files {
+		fileLine := !conf.NoFileLine
+		ctx := &blockCtx{
+			pkg: p, pkgCtx: ctx, cb: p.CB(), fset: p.Fset, targetDir: targetDir,
+			fileLine: fileLine, relativePath: conf.RelativePath, isClass: f.IsClass,
+			c2goBase: c2goBase(conf.C2goBase), imports: make(map[string]*gox.PkgRef),
+		}
+		preloadGopFile(p, ctx, fpath, f, conf)
 	}
-	for _, f := range pkg.Files {
-		if f.FileType == ast.FileTypeGmx {
+	for fpath, gof := range pkg.GoFiles {
+		f := fromgo.ASTFile(gof, 0)
+		ctx := &blockCtx{
+			pkg: p, pkgCtx: ctx, cb: p.CB(), fset: p.Fset, targetDir: targetDir,
+			imports: make(map[string]*gox.PkgRef),
+		}
+		preloadFile(p, ctx, fpath, f, false)
+	}
+	for _, f := range files {
+		if f.IsProj {
 			loadFile(ctx, f)
 			gmxMainFunc(p, ctx)
 			break
 		}
 	}
-	for _, f := range pkg.Files {
-		if f.FileType != ast.FileTypeGmx { // only one .gmx file
+	for _, f := range files {
+		if !f.IsProj { // only one .gmx file
 			loadFile(ctx, f)
 		}
 	}
@@ -459,6 +447,14 @@ func NewPackage(pkgPath string, pkg *ast.Package, conf *Config) (p *gox.Package,
 		load()
 	}
 	err = ctx.complete()
+
+	if !conf.NoAutoGenMain && pkg.Name == "main" {
+		if obj := p.Types.Scope().Lookup("main"); obj == nil {
+			old, _ := p.SetCurFile(defaultGoFile, false)
+			p.NewFunc(nil, "main", nil, nil, false).BodyStart(p).End()
+			p.RestoreCurFile(old)
+		}
+	}
 	return
 }
 
@@ -475,16 +471,15 @@ func hasMethod(o types.Object, name string) bool {
 	return false
 }
 
-func getEntrypoint(ft ast.FileType, isMod bool) string {
-	switch ft {
-	case ast.FileTypeSpx:
-		return "Main"
-	case ast.FileTypeGmx:
+func getEntrypoint(f *ast.File, isMod bool) string {
+	switch {
+	case f.IsProj:
 		return "MainEntry"
+	case f.IsClass:
+		return "Main"
+	case isMod:
+		return "init"
 	default:
-		if isMod {
-			return "init"
-		}
 		return "main"
 	}
 }
@@ -520,32 +515,37 @@ func loadFile(ctx *pkgCtx, f *ast.File) {
 	}
 }
 
-func preloadFile(p *gox.Package, parent *pkgCtx, file string, f *ast.File, targetDir string, conf *Config) {
-	syms := parent.syms
-	fileLine := !conf.NoFileLine
-	testingFile := strings.HasSuffix(file, "_test.gop")
-	ctx := &blockCtx{
-		pkg: p, pkgCtx: parent, cb: p.CB(), fset: p.Fset, targetDir: targetDir, fileType: f.FileType,
-		fileLine: fileLine, relativePath: conf.RelativePath, imports: make(map[string]*gox.PkgRef),
+func getGoFile(file string, genCode bool) string {
+	if genCode {
+		if strings.HasSuffix(file, "_test.gop") {
+			return testingGoFile
+		}
+		return defaultGoFile
 	}
+	return skippingGoFile
+}
+
+func preloadGopFile(p *gox.Package, ctx *blockCtx, file string, f *ast.File, conf *Config) {
+	var parent = ctx.pkgCtx
 	var classType string
 	var baseTypeName string
 	var baseType types.Type
-	switch f.FileType {
-	case ast.FileTypeSpx:
-		if parent.gmxSettings != nil {
-			classType = getDefaultClass(file)
-			o := parent.sprite
-			baseTypeName, baseType = o.Name(), o.Type()
-		}
-		// TODO: panic
-	case ast.FileTypeGmx:
+	var spxClass bool
+	switch {
+	case f.IsProj:
 		classType = parent.gameClass
 		o := parent.game
 		baseTypeName, baseType = o.Name(), o.Type()
 		if parent.gameIsPtr {
 			baseType = types.NewPointer(baseType)
 		}
+	case f.IsClass:
+		if parent.gmxSettings != nil {
+			classType = getDefaultClass(file)
+			o := parent.sprite
+			baseTypeName, baseType, spxClass = o.Name(), o.Type(), true
+		}
+		// TODO: panic
 	}
 	if classType != "" {
 		if debugLoad {
@@ -555,6 +555,7 @@ func preloadFile(p *gox.Package, parent *pkgCtx, file string, f *ast.File, targe
 		for i, pkgPath := range parent.pkgPaths {
 			ctx.lookups[i] = p.Import(pkgPath)
 		}
+		syms := parent.syms
 		pos := f.Pos()
 		specs := getFields(ctx, f)
 		ld := getTypeLoader(parent, syms, pos, classType)
@@ -570,7 +571,7 @@ func preloadFile(p *gox.Package, parent *pkgCtx, file string, f *ast.File, targe
 				pkg := p.Types
 				flds := make([]*types.Var, 1, 2)
 				flds[0] = types.NewField(pos, pkg, baseTypeName, baseType, true)
-				if f.FileType == ast.FileTypeSpx {
+				if spxClass {
 					typ := toType(ctx, &ast.StarExpr{X: &ast.Ident{Name: parent.gameClass}})
 					fld := types.NewField(pos, pkg, getTypeName(typ), typ, true)
 					flds = append(flds, fld)
@@ -604,11 +605,39 @@ func preloadFile(p *gox.Package, parent *pkgCtx, file string, f *ast.File, targe
 			},
 		}}}
 	}
+	// check class project no MainEntry and auto added
+	if f.IsProj && !conf.NoAutoGenMain && !f.NoEntrypoint && f.Name.Name == "main" {
+		entry := getEntrypoint(f, false)
+		var hasEntry bool
+		for _, decl := range f.Decls {
+			switch d := decl.(type) {
+			case *ast.FuncDecl:
+				if d.Name.Name == entry {
+					hasEntry = true
+				}
+			}
+		}
+		if !hasEntry {
+			f.Decls = append(f.Decls, &ast.FuncDecl{
+				Name: ast.NewIdent(entry),
+				Type: &ast.FuncType{Params: &ast.FieldList{}},
+				Body: &ast.BlockStmt{},
+			})
+		}
+	}
+	preloadFile(p, ctx, file, f, true)
+}
+
+func preloadFile(p *gox.Package, ctx *blockCtx, file string, f *ast.File, genCode bool) {
+	parent := ctx.pkgCtx
+	syms := parent.syms
+	goFile := getGoFile(file, genCode)
+	p.SetCurFile(goFile, true)
 	for _, decl := range f.Decls {
 		switch d := decl.(type) {
 		case *ast.FuncDecl:
 			if f.NoEntrypoint && d.Name.Name == "main" {
-				d.Name.Name = getEntrypoint(f.FileType, f.Name.Name != "main")
+				d.Name.Name = getEntrypoint(f, f.Name.Name != "main")
 			}
 			if ctx.classRecv != nil { // in class file (.spx/.gmx)
 				if d.Recv == nil {
@@ -616,42 +645,60 @@ func preloadFile(p *gox.Package, parent *pkgCtx, file string, f *ast.File, targe
 				}
 			}
 			if d.Recv == nil {
-				name := d.Name
-				fn := func() {
-					old := p.SetInTestingFile(testingFile)
-					defer p.SetInTestingFile(old)
-					loadFunc(ctx, nil, d)
+				var name = d.Name
+				var fn func()
+				if genCode {
+					fn = func() {
+						old, _ := p.SetCurFile(goFile, true)
+						defer p.RestoreCurFile(old)
+						loadFunc(ctx, nil, d)
+					}
+				} else {
+					fn = func() {
+						declFunc(ctx, nil, d)
+					}
 				}
 				if name.Name == "init" {
-					if debugLoad {
-						log.Println("==> Preload func init")
+					if genCode {
+						if debugLoad {
+							log.Println("==> Preload func init")
+						}
+						parent.inits = append(parent.inits, fn)
 					}
-					parent.inits = append(parent.inits, fn)
 				} else {
 					if debugLoad {
 						log.Println("==> Preload func", name.Name)
 					}
-					initLoader(parent, syms, name.Pos(), name.Name, fn)
+					initLoader(parent, syms, name.Pos(), name.Name, fn, genCode)
 				}
 			} else {
 				if name, ok := getRecvTypeName(parent, d.Recv, true); ok {
 					if debugLoad {
 						log.Printf("==> Preload method %s.%s\n", name, d.Name.Name)
 					}
-					ld := getTypeLoader(parent, syms, token.NoPos, name)
-					ld.methods = append(ld.methods, func() {
-						old := p.SetInTestingFile(testingFile)
-						defer p.SetInTestingFile(old)
-						doInitType(ld)
-						recv := toRecv(ctx, d.Recv)
-						loadFunc(ctx, recv, d)
-					})
+					var ld = getTypeLoader(parent, syms, token.NoPos, name)
+					var fn func()
+					if genCode {
+						fn = func() {
+							old, _ := p.SetCurFile(goFile, true)
+							defer p.RestoreCurFile(old)
+							doInitType(ld)
+							recv := toRecv(ctx, d.Recv)
+							loadFunc(ctx, recv, d)
+						}
+					} else {
+						fn = func() {
+							doInitType(ld)
+							recv := toRecv(ctx, d.Recv)
+							declFunc(ctx, recv, d)
+						}
+					}
+					ld.methods = append(ld.methods, fn)
 				}
 			}
 		case *ast.GenDecl:
 			switch d.Tok {
 			case token.IMPORT:
-				p.SetInTestingFile(testingFile)
 				for _, item := range d.Specs {
 					loadImport(ctx, item.(*ast.ImportSpec))
 				}
@@ -663,30 +710,53 @@ func preloadFile(p *gox.Package, parent *pkgCtx, file string, f *ast.File, targe
 						log.Println("==> Preload type", name)
 					}
 					ld := getTypeLoader(parent, syms, t.Name.Pos(), name)
-					ld.typ = func() {
-						old := p.SetInTestingFile(testingFile)
-						defer p.SetInTestingFile(old)
-						if t.Assign != token.NoPos { // alias type
-							if debugLoad {
-								log.Println("==> Load > AliasType", name)
+					if genCode {
+						ld.typ = func() {
+							old, _ := p.SetCurFile(goFile, true)
+							defer p.RestoreCurFile(old)
+							if t.Assign != token.NoPos { // alias type
+								if debugLoad {
+									log.Println("==> Load > AliasType", name)
+								}
+								ctx.pkg.AliasType(name, toType(ctx, t.Type), t.Pos())
+								return
 							}
-							ctx.pkg.AliasType(name, toType(ctx, t.Type))
-							return
-						}
-						if debugLoad {
-							log.Println("==> Load > NewType", name)
-						}
-						decl := ctx.pkg.NewType(name)
-						if t.Doc != nil {
-							decl.SetComments(t.Doc)
-						} else if d.Doc != nil {
-							decl.SetComments(d.Doc)
-						}
-						ld.typInit = func() { // decycle
 							if debugLoad {
-								log.Println("==> Load > InitType", name)
+								log.Println("==> Load > NewType", name)
+							}
+							decl := ctx.pkg.NewType(name)
+							if t.Doc != nil {
+								decl.SetComments(t.Doc)
+							} else if d.Doc != nil {
+								decl.SetComments(d.Doc)
+							}
+							ld.typInit = func() { // decycle
+								if debugLoad {
+									log.Println("==> Load > InitType", name)
+								}
+								decl.InitType(ctx.pkg, toType(ctx, t.Type))
+							}
+						}
+					} else {
+						ld.typ = func() {
+							pkg := ctx.pkg.Types
+							if t.Assign != token.NoPos { // alias type
+								if debugLoad {
+									log.Println("==> Load > AliasType", name)
+								}
+								aliasType(pkg, t.Pos(), name, toType(ctx, t.Type))
+								return
+							}
+							if debugLoad {
+								log.Println("==> Load > NewType", name)
+							}
+							named := newType(pkg, t.Pos(), name)
+							ld.typInit = func() { // decycle
+								if debugLoad {
+									log.Println("==> Load > InitType", name)
+								}
+								initType(ctx, named, toType(ctx, t.Type))
 							}
-							decl.InitType(ctx.pkg, toType(ctx, t.Type))
 						}
 					}
 				}
@@ -700,8 +770,6 @@ func preloadFile(p *gox.Package, parent *pkgCtx, file string, f *ast.File, targe
 					}
 					setNamesLoader(parent, syms, vSpec.Names, func() {
 						if c := cdecl; c != nil {
-							old := p.SetInTestingFile(testingFile)
-							defer p.SetInTestingFile(old)
 							cdecl = nil
 							loadConstSpecs(ctx, c, d.Specs)
 							for _, s := range d.Specs {
@@ -719,9 +787,9 @@ func preloadFile(p *gox.Package, parent *pkgCtx, file string, f *ast.File, targe
 					}
 					setNamesLoader(parent, syms, vSpec.Names, func() {
 						if v := vSpec; v != nil { // only init once
-							old := p.SetInTestingFile(testingFile)
-							defer p.SetInTestingFile(old)
 							vSpec = nil
+							old, _ := p.SetCurFile(goFile, true)
+							defer p.RestoreCurFile(old)
 							loadVars(ctx, v, true)
 							removeNames(syms, v.Names)
 						}
@@ -736,6 +804,59 @@ func preloadFile(p *gox.Package, parent *pkgCtx, file string, f *ast.File, targe
 	}
 }
 
+func newType(pkg *types.Package, pos token.Pos, name string) *types.Named {
+	typName := types.NewTypeName(pos, pkg, name, nil)
+	if old := pkg.Scope().Insert(typName); old != nil {
+		log.Panicf("%s redeclared in this block\n\tprevious declaration at %v\n", name, "<TODO>")
+	}
+	return types.NewNamed(typName, nil, nil)
+}
+
+func initType(ctx *blockCtx, named *types.Named, typ types.Type) {
+	if named, ok := typ.(*types.Named); ok {
+		typ = getUnderlying(ctx, named)
+	}
+	named.SetUnderlying(typ)
+}
+
+func aliasType(pkg *types.Package, pos token.Pos, name string, typ types.Type) {
+	o := types.NewTypeName(pos, pkg, name, typ)
+	pkg.Scope().Insert(o)
+}
+
+func declFunc(ctx *blockCtx, recv *types.Var, d *ast.FuncDecl) {
+	name := d.Name.Name
+	if debugLoad {
+		if recv == nil {
+			log.Println("==> Load func", name)
+		} else {
+			log.Printf("==> Load method %v.%s\n", recv.Type(), name)
+		}
+	}
+	if name == "_" {
+		return
+	}
+	pkg := ctx.pkg.Types
+	sig := toFuncType(ctx, d.Type, recv)
+	fn := types.NewFunc(d.Pos(), pkg, name, sig)
+	if recv != nil {
+		typ := recv.Type()
+		switch t := typ.(type) {
+		case *types.Named:
+			t.AddMethod(fn)
+			return
+		case *types.Pointer:
+			if tt, ok := t.Elem().(*types.Named); ok {
+				tt.AddMethod(fn)
+				return
+			}
+		}
+		log.Panicf("invalid receiver type %v (%v is not a defined type)\n", typ, typ)
+	} else {
+		pkg.Scope().Insert(fn)
+	}
+}
+
 func loadFunc(ctx *blockCtx, recv *types.Var, d *ast.FuncDecl) {
 	name := d.Name.Name
 	if debugLoad {
@@ -809,7 +930,6 @@ var binaryGopNames = map[string]string{
 	"<<=": "Gop_LshAssign",
 	">>=": "Gop_RshAssign",
 	"&^=": "Gop_AndNotAssign",
-	"=":   "Gop_Assign",
 
 	"==": "Gop_EQ",
 	"!=": "Gop_NE",
@@ -828,7 +948,7 @@ var unaryGopNames = map[string]string{
 	"++": "Gop_Inc",
 	"--": "Gop_Dec",
 	"-":  "Gop_Neg",
-	"+":  "Gop_Pos",
+	"+":  "Gop_Dup",
 	"^":  "Gop_Not",
 	"!":  "Gop_LNot",
 	"<-": "Gop_Recv",
@@ -848,8 +968,18 @@ func simplifyGopPackage(pkgPath string) string {
 }
 
 func loadImport(ctx *blockCtx, spec *ast.ImportSpec) {
-	pkgPath := toString(spec.Path)
-	pkg := ctx.pkg.Import(simplifyGopPackage(pkgPath))
+	var pkg *gox.PkgRef
+	var pkgPath = toString(spec.Path)
+	if realPath, kind := checkC2go(pkgPath); kind != c2goInvalid {
+		if kind == c2goStandard {
+			realPath = ctx.c2goBase + realPath
+		}
+		if pkg = loadC2goPkg(ctx, realPath, spec.Path); pkg == nil {
+			return
+		}
+	} else {
+		pkg = ctx.pkg.Import(simplifyGopPackage(pkgPath))
+	}
 	var name string
 	if spec.Name != nil {
 		name = spec.Name.Name
@@ -929,6 +1059,10 @@ func loadVars(ctx *blockCtx, v *ast.ValueSpec, global bool) {
 					} else {
 						panic(ctx.newCodeErrorf(e.Pos(), "lambda unsupport multiple assignment"))
 					}
+				case *ast.SliceLit:
+					compileSliceLit(ctx, e, typ)
+				case *ast.CompositeLit:
+					compileCompositeLit(ctx, e, typ, false)
 				default:
 					compileExpr(ctx, val)
 				}
@@ -954,7 +1088,7 @@ func removeNames(syms map[string]loader, names []*ast.Ident) {
 
 func setNamesLoader(ctx *pkgCtx, syms map[string]loader, names []*ast.Ident, load func()) {
 	for _, name := range names {
-		initLoader(ctx, syms, name.Pos(), name.Name, load)
+		initLoader(ctx, syms, name.Pos(), name.Name, load, true)
 	}
 }
 
