@@ -21,65 +21,79 @@ import (
 	"os"
 	"strings"
 	"sync"
-	"syscall"
 	"testing"
 
+	"github.com/goplus/gop"
 	"github.com/goplus/gop/cl"
-	"github.com/goplus/gop/cmd/gengo"
 	"github.com/goplus/gop/parser"
 	"github.com/goplus/gop/parser/parsertest"
 	"github.com/goplus/gop/scanner"
 	"github.com/goplus/gop/token"
 	"github.com/goplus/gox"
+	"github.com/goplus/mod/env"
+)
+
+const (
+	gopRootDir = ".."
 )
 
 var (
-	gblFset  *token.FileSet
-	baseConf *cl.Config
+	gblFset *token.FileSet
+	gblConf *cl.Config
 )
 
 func init() {
 	gox.SetDebug(gox.DbgFlagAll)
 	cl.SetDebug(cl.DbgFlagAll)
 	gblFset = token.NewFileSet()
-	baseConf = &cl.Config{
+	imp := gop.NewImporter(nil, &env.Gop{Root: gopRootDir, Version: "1.0"}, gblFset)
+	gblConf = &cl.Config{
 		Fset:          gblFset,
-		ModRootDir:    "./internal",
-		GenGoPkg:      new(gengo.Runner).GenGoPkg,
-		CacheLoadPkgs: true,
+		Importer:      imp,
+		LookupClass:   lookupClass,
+		LookupPub:     lookupPub,
+		C2goBase:      "github.com/goplus/gop/cl/internal",
 		NoFileLine:    true,
+		NoAutoGenMain: true,
 	}
 }
 
-func gopClTest(t *testing.T, gopcode, expected string, cachefile ...string) {
-	gopClTestEx(t, "main", gopcode, expected, cachefile...)
+func gopClNamedTest(t *testing.T, name string, gopcode, expected string) {
+	t.Run(name, func(t *testing.T) {
+		gopClTest(t, gopcode, expected)
+	})
 }
 
-func gopClTestEx(t *testing.T, pkgname, gopcode, expected string, cachefile ...string) {
+func gopClTest(t *testing.T, gopcode, expected string) {
+	gopClTestEx(t, gblConf, "main", gopcode, expected)
+}
+
+func gopClTestEx(t *testing.T, conf *cl.Config, pkgname, gopcode, expected string) {
+	fs := parsertest.NewSingleFileFS("/foo", "bar.gop", gopcode)
+	gopClTestFS(t, conf, fs, pkgname, expected)
+}
+
+func gopMixedClTest(t *testing.T, pkgname, gocode, gopcode, expected string) {
+	fs := parsertest.NewTwoFilesFS("/foo", "a.go", gocode, "b.gop", gopcode)
+	gopClTestFS(t, gblConf, fs, pkgname, expected)
+}
+
+func gopClTestFS(t *testing.T, conf *cl.Config, fs parser.FileSystem, pkgname, expected string) {
 	cl.SetDisableRecover(true)
 	defer cl.SetDisableRecover(false)
 
-	fs := parsertest.NewSingleFileFS("/foo", "bar.gop", gopcode)
-	pkgs, err := parser.ParseFSDir(gblFset, fs, "/foo", nil, parser.ParseComments)
+	pkgs, err := parser.ParseFSDir(gblFset, fs, "/foo", parser.Config{Mode: parser.ParseComments})
 	if err != nil {
 		scanner.PrintError(os.Stderr, err)
 		t.Fatal("ParseFSDir:", err)
 	}
-	conf := *baseConf.Ensure()
-	if cachefile != nil {
-		copy := *baseConf
-		copy.PkgsLoader = nil
-		copy.CacheFile = cachefile[0]
-		copy.PersistLoadPkgs = true
-		conf = *copy.Ensure()
-	}
 	bar := pkgs[pkgname]
-	pkg, err := cl.NewPackage("github.com/goplus/gop/cl", bar, &conf)
+	pkg, err := cl.NewPackage("github.com/goplus/gop/cl", bar, conf)
 	if err != nil {
 		t.Fatal("NewPackage:", err)
 	}
 	var b bytes.Buffer
-	err = gox.WriteTo(&b, pkg, false)
+	err = pkg.WriteTo(&b)
 	if err != nil {
 		t.Fatal("gox.WriteTo failed:", err)
 	}
@@ -87,33 +101,266 @@ func gopClTestEx(t *testing.T, pkgname, gopcode, expected string, cachefile ...s
 	if result != expected {
 		t.Fatalf("\nResult:\n%s\nExpected:\n%s\n", result, expected)
 	}
-	if cachefile != nil {
-		if err = conf.PkgsLoader.Save(); err != nil {
-			t.Fatal("PkgsLoader.Save failed:", err)
+}
+
+func TestMixedGo(t *testing.T) {
+	gopMixedClTest(t, "main", `package main
+
+import "strconv"
+
+const n = 10
+
+func f(v int) string {
+	return strconv.Itoa(v)
+}
+
+type foo struct {
+	v int
+}
+
+func (a foo) _() {
+}
+
+func (a foo) Str() string {
+	return f(a.v)
+}
+
+func (a *foo) Bar() int {
+	return 0
+}
+
+type foo2 = foo
+type foo3 foo2
+`, `
+var a [n]int
+var b string = f(n)
+var c foo2
+var d int = c.v
+var e = foo3{}
+var x string = c.str
+`, `package main
+
+var a [10]int
+var b string = f(n)
+var c foo
+var d int = c.v
+var e = foo3{}
+var x string = c.Str()
+`)
+}
+
+func Test_RangeExpressionIf_Issue1243(t *testing.T) {
+	gopClTest(t, `
+for i <- :10, i%3 == 0 {
+	println i
+}`, `package main
+
+import fmt "fmt"
+
+func main() {
+	for i := 0; i < 10; i += 1 {
+		if i%3 == 0 {
+			fmt.Println(i)
 		}
 	}
 }
-
-func TestEmptyPkgsLoader(t *testing.T) {
-	l := &cl.PkgsLoader{}
-	if l.Save() != nil {
-		t.Fatal("PkgsLoader.Save failed")
-	}
-	if l.GenGoPkgs(nil, nil) != syscall.ENOENT {
-		t.Fatal("PkgsLoader.GenGoPkgs failed")
-	}
-	if _, err := cl.GetModulePath("/dir-not-exists/go.mod"); err == nil {
-		t.Fatal("GetModulePath failed")
-	}
+`)
 }
 
-func TestNilConf(t *testing.T) {
-	fs := parsertest.NewSingleFileFS("/foo", "bar.gop", `println("Hi")`)
-	pkgs, err := parser.ParseFSDir(gblFset, fs, "/foo", nil, 0)
-	if err != nil {
-		t.Fatal("ParseFSDir:", err)
-	}
-	cl.NewPackage("", pkgs["main"], nil)
+func Test_CastSlice_Issue1240(t *testing.T) {
+	gopClTest(t, `
+type fvec []float64
+type foo float64
+a := []float64([1, 2])
+b := fvec([1, 2])
+c := foo([1, 2])
+d := fvec([])
+println a, b, c, d
+`, `package main
+
+import fmt "fmt"
+
+type fvec []float64
+type foo float64
+
+func main() {
+	a := []float64{1, 2}
+	b := fvec{1, 2}
+	c := foo([]int{1, 2})
+	d := fvec{}
+	fmt.Println(a, b, c, d)
+}
+`)
+}
+
+func TestUnderscoreRedeclared_Issue1197(t *testing.T) {
+	gopClTest(t, `
+func() (_ [2]int) { type _ int; return }()
+`, `package main
+
+func main() {
+	func() (_ [2]int) {
+		return
+	}()
+}
+`)
+}
+
+func TestInterfaceBugNilUnderlying_Issue1198(t *testing.T) {
+	gopClTest(t, `
+import "runtime"
+
+type Outer interface{ Inner }
+
+type impl struct{}
+
+func New() Outer { return &impl{} }
+
+type Inner interface {
+	DoStuff() error
+}
+
+func (a *impl) DoStuff() error {
+	return nil
+}
+
+func main() {
+	var outer Outer = New()
+}
+`, `package main
+
+type Inner interface {
+	DoStuff() error
+}
+type Outer interface {
+	Inner
+}
+type impl struct {
+}
+
+func (a *impl) DoStuff() error {
+	return nil
+}
+func New() Outer {
+	return &impl{}
+}
+func main() {
+	var outer Outer = New()
+}
+`)
+}
+
+func TestInterfaceBugNilUnderlying_Issue1196(t *testing.T) {
+	gopClTest(t, `
+func main() {
+	i := I(A{})
+
+	b := make(chan I, 1)
+	b <- B{}
+
+	var ok bool
+	i, ok = <-b
+}
+
+type I interface{ M() int }
+
+type T int
+
+func (T) M() int { return 0 }
+
+type A struct{ T }
+type B struct{ T }
+`, `package main
+
+func main() {
+	i := I(A{})
+	b := make(chan I, 1)
+	b <- B{}
+	var ok bool
+	i, ok = <-b
+}
+
+type T int
+
+func (T) M() int {
+	return 0
+}
+
+type A struct {
+	T
+}
+type I interface {
+	M() int
+}
+type B struct {
+	T
+}
+`)
+}
+
+func TestMyIntInc_Issue1195(t *testing.T) {
+	gopClTest(t, `
+type MyInt int
+var c MyInt
+c++
+`, `package main
+
+type MyInt int
+
+var c MyInt
+
+func main() {
+	c++
+}
+`)
+}
+
+func TestAutoPropMixedName_Issue1194(t *testing.T) {
+	gopClTest(t, `
+type Point struct {
+	Min, Max int
+}
+
+type Obj struct {
+	bbox Point
+}
+
+func (o *Obj) Bbox() Point {
+	return o.bbox
+}
+
+func (o *Obj) Points() [2]int{
+	return [2]int{o.bbox.Min, o.bbox.Max}
+}
+`, `package main
+
+type Point struct {
+	Min int
+	Max int
+}
+type Obj struct {
+	bbox Point
+}
+
+func (o *Obj) Bbox() Point {
+	return o.bbox
+}
+func (o *Obj) Points() [2]int {
+	return [2]int{o.bbox.Min, o.bbox.Max}
+}
+`)
+}
+
+func TestShiftUntypedInt_Issue1193(t *testing.T) {
+	gopClTest(t, `
+func GetValue(shift uint) uint {
+	return 1 << shift
+}`, `package main
+
+func GetValue(shift uint) uint {
+	return 1 << shift
+}
+`)
 }
 
 func TestInitFunc(t *testing.T) {
@@ -175,14 +422,14 @@ const (
 )
 `, `package main
 
-func main() {
-	const a = append + len
-}
-
 const (
 	append = iota
 	len
 )
+
+func main() {
+	const a = append + len
+}
 `)
 }
 
@@ -567,7 +814,7 @@ func main() {
 }
 
 func TestIssue774(t *testing.T) {
-	gopClTest(t, `
+	gopClNamedTest(t, "InterfaceTypeAssert", `
 package main
 
 import "fmt"
@@ -597,9 +844,6 @@ func main() {
 	fmt.Println(a.(*A))
 }
 
-type AA interface {
-	String() string
-}
 type A struct {
 	str string
 }
@@ -607,8 +851,12 @@ type A struct {
 func (a *A) String() string {
 	return a.str
 }
+
+type AA interface {
+	String() string
+}
 `)
-	gopClTest(t, `
+	gopClNamedTest(t, "getInterface", `
 package main
 
 import "fmt"
@@ -642,16 +890,14 @@ func main() {
 	a := get()
 	fmt.Println(a.(*A))
 }
-
-type AA interface {
-	String() string
-}
-
 func get() AA {
 	var a AA
 	return a
 }
 
+type AA interface {
+	String() string
+}
 type A struct {
 	str string
 }
@@ -1222,16 +1468,42 @@ var x, y = 1, "Hi"
 `)
 }
 
+func TestUint128Add(t *testing.T) {
+	gopClTest(t, `
+var x, y uint128
+var z uint128 = x + y
+`, `package main
+
+import ng "github.com/goplus/gop/builtin/ng"
+
+var x, y ng.Uint128
+var z ng.Uint128 = x.Gop_Add__1(y)
+`)
+}
+
+func TestInt128Add(t *testing.T) {
+	gopClTest(t, `
+var x, y int128
+var z int128 = x + y
+`, `package main
+
+import ng "github.com/goplus/gop/builtin/ng"
+
+var x, y ng.Int128
+var z ng.Int128 = x.Gop_Add__1(y)
+`)
+}
+
 func TestBigIntAdd(t *testing.T) {
 	gopClTest(t, `
 var x, y bigint
 var z bigint = x + y
 `, `package main
 
-import builtin "github.com/goplus/gop/builtin"
+import ng "github.com/goplus/gop/builtin/ng"
 
-var x, y builtin.Gop_bigint
-var z builtin.Gop_bigint = x.Gop_Add(y)
+var x, y ng.Bigint
+var z ng.Bigint = x.Gop_Add(y)
 `)
 }
 
@@ -1241,11 +1513,33 @@ var x = 1r
 `, `package main
 
 import (
-	builtin "github.com/goplus/gop/builtin"
+	ng "github.com/goplus/gop/builtin/ng"
 	big "math/big"
 )
 
-var x = builtin.Gop_bigint_Init__1(big.NewInt(1))
+var x = ng.Bigint_Init__1(big.NewInt(1))
+`)
+}
+
+func TestUint128Lit(t *testing.T) {
+	gopClTest(t, `
+var x uint128 = 1
+`, `package main
+
+import ng "github.com/goplus/gop/builtin/ng"
+
+var x ng.Uint128 = ng.Uint128_Init__0(1)
+`)
+}
+
+func TestInt128Lit(t *testing.T) {
+	gopClTest(t, `
+var x int128 = 1
+`, `package main
+
+import ng "github.com/goplus/gop/builtin/ng"
+
+var x ng.Int128 = ng.Int128_Init__0(1)
 `)
 }
 
@@ -1255,11 +1549,11 @@ var x = 1/2r
 `, `package main
 
 import (
-	builtin "github.com/goplus/gop/builtin"
+	ng "github.com/goplus/gop/builtin/ng"
 	big "math/big"
 )
 
-var x = builtin.Gop_bigrat_Init__2(big.NewRat(1, 2))
+var x = ng.Bigrat_Init__2(big.NewRat(1, 2))
 `)
 }
 
@@ -1269,11 +1563,11 @@ var x = 3 + 1/2r
 `, `package main
 
 import (
-	builtin "github.com/goplus/gop/builtin"
+	ng "github.com/goplus/gop/builtin/ng"
 	big "math/big"
 )
 
-var x = builtin.Gop_bigrat_Init__2(big.NewRat(7, 2))
+var x = ng.Bigrat_Init__2(big.NewRat(7, 2))
 `)
 }
 
@@ -1286,13 +1580,13 @@ var z = 100 + y
 `, `package main
 
 import (
-	builtin "github.com/goplus/gop/builtin"
+	ng "github.com/goplus/gop/builtin/ng"
 	big "math/big"
 )
 
-var x = builtin.Gop_bigrat_Init__2(big.NewRat(7, 2))
-var y = x.Gop_Add(builtin.Gop_bigrat_Init__0(100))
-var z = builtin.Gop_bigrat_Init__0(100) + y
+var x = ng.Bigrat_Init__2(big.NewRat(7, 2))
+var y = x.Gop_Add(ng.Bigrat_Init__0(100))
+var z = ng.Bigrat_Init__0(100) + y
 `)
 }
 
@@ -1374,12 +1668,12 @@ var x bigint
 x += 3
 `, `package main
 
-import builtin "github.com/goplus/gop/builtin"
+import ng "github.com/goplus/gop/builtin/ng"
 
-var x builtin.Gop_bigint
+var x ng.Bigint
 
 func main() {
-	x.Gop_AddAssign(builtin.Gop_bigint_Init__0(3))
+	x.Gop_AddAssign(ng.Bigint_Init__0(3))
 }
 `)
 }
@@ -1391,13 +1685,13 @@ x *= 2
 `, `package main
 
 import (
-	builtin "github.com/goplus/gop/builtin"
+	ng "github.com/goplus/gop/builtin/ng"
 	big "math/big"
 )
 
 func main() {
-	x := builtin.Gop_bigint_Init__1(big.NewInt(3))
-	x.Gop_MulAssign(builtin.Gop_bigint_Init__0(2))
+	x := ng.Bigint_Init__1(big.NewInt(3))
+	x.Gop_MulAssign(ng.Bigint_Init__0(2))
 }
 `)
 }
@@ -1409,13 +1703,13 @@ x *= 2r
 `, `package main
 
 import (
-	builtin "github.com/goplus/gop/builtin"
+	ng "github.com/goplus/gop/builtin/ng"
 	big "math/big"
 )
 
 func main() {
-	x := builtin.Gop_bigint_Init__1(big.NewInt(3))
-	x.Gop_MulAssign(builtin.Gop_bigint_Init__1(big.NewInt(2)))
+	x := ng.Bigint_Init__1(big.NewInt(3))
+	x.Gop_MulAssign(ng.Bigint_Init__1(big.NewInt(2)))
 }
 `)
 }
@@ -1532,6 +1826,22 @@ func foo() *Result {
 `)
 }
 
+func TestCompositeLit5(t *testing.T) {
+	gopClTest(t, `
+type mymap map[float64]string
+var x = {1:"hello", 2:"world"}
+var y map[float64]string = {1:"hello", 2:"world"}
+var z mymap = {1:"hello", 2:"world"}
+`, `package main
+
+type mymap map[float64]string
+
+var x = map[int]string{1: "hello", 2: "world"}
+var y map[float64]string = map[float64]string{1: "hello", 2: "world"}
+var z mymap = mymap{1: "hello", 2: "world"}
+`)
+}
+
 func TestSliceLit(t *testing.T) {
 	gopClTest(t, `
 x := [1, 3.4, 5]
@@ -1545,6 +1855,19 @@ func main() {
 	z := []interface {
 	}{}
 }
+`)
+	gopClTest(t, `
+type vector []float64
+var x = [1, 2, 3]
+var y []float64 = [1, 2, 3]
+var z vector = [1, 2, 3]
+`, `package main
+
+type vector []float64
+
+var x = []int{1, 2, 3}
+var y []float64 = []float64{1, 2, 3}
+var z vector = vector{1, 2, 3}
 `)
 }
 
@@ -1628,12 +1951,12 @@ func main() {
 }
 `, `package main
 
+type bar = foo
 type foo struct {
 	p *foo
 	A int
 	B string "tag1:123"
 }
-type bar = foo
 
 func main() {
 	type a struct {
@@ -1903,10 +2226,6 @@ type fooIter struct {
 	data *foo
 	idx  int
 }
-type foo struct {
-	key []int
-	val []string
-}
 
 func (p *fooIter) Next() (key int, val string, ok bool) {
 	if p.idx < len(p.data.key) {
@@ -1915,6 +2234,12 @@ func (p *fooIter) Next() (key int, val string, ok bool) {
 	}
 	return
 }
+
+type foo struct {
+	key []int
+	val []string
+}
+
 func (p *foo) Gop_Enum() *fooIter {
 	return &fooIter{data: p}
 }
@@ -2973,6 +3298,39 @@ func main() {
 `)
 }
 
+func TestLambdaExpr3(t *testing.T) {
+	gopClTest(t, `
+func intSeq() func() int {
+	i := 0
+	return => {
+		i++
+		return i
+	}
+}
+`, `package main
+
+func intSeq() func() int {
+	i := 0
+	return func() int {
+		i++
+		return i
+	}
+}
+`)
+	gopClTest(t, `
+func intDouble() func(int) int {
+	return i => i*2
+}
+`, `package main
+
+func intDouble() func(int) int {
+	return func(i int) int {
+		return i * 2
+	}
+}
+`)
+}
+
 func TestUnnamedMainFunc(t *testing.T) {
 	gopClTest(t, `i := 1`, `package main
 
@@ -3194,15 +3552,6 @@ func main() {
 }
 
 func TestCallDep(t *testing.T) {
-	const (
-		cachedir  = "../.gop"
-		cachefile = cachedir + "/gop.cache"
-	)
-	os.Remove(cachefile)
-	defer func() {
-		os.Remove(cachefile)
-		os.Remove(cachedir)
-	}()
 	for i := 0; i < 2; i++ {
 		gopClTest(t, `
 import (
@@ -3248,24 +3597,21 @@ func TestNew(t *testing.T) {
 		t.Fatal("Test failed:", ret, expected)
 	}
 }
-
-type Result struct {
-	Repo Repo
-}
-
 func New() Result {
 	repo := newRepo()
 	return Result{Repo: repo}
 }
+func newRepo() Repo {
+	return Repo{Title: "Hi"}
+}
 
 type Repo struct {
 	Title string
 }
-
-func newRepo() Repo {
-	return Repo{Title: "Hi"}
+type Result struct {
+	Repo Repo
 }
-`, "")
+`)
 	}
 }
 
@@ -3534,7 +3880,7 @@ func main() {
 	fmt.Println("init")
 }
 `)
-	gopClTestEx(t, "bar", `package bar
+	gopClTestEx(t, gblConf, "bar", `package bar
 println("init")
 `, `package bar
 
@@ -3621,3 +3967,59 @@ func main() {
 }
 `)
 }
+
+func TestAnyAlias(t *testing.T) {
+	gopClTest(t, `
+var a any = 100
+println(a)
+`, `package main
+
+import fmt "fmt"
+
+var a interface {
+} = 100
+
+func main() {
+	fmt.Println(a)
+}
+`)
+}
+
+func TestMainEntry(t *testing.T) {
+	conf := *gblConf
+	conf.NoAutoGenMain = false
+
+	gopClTestEx(t, &conf, "main", `
+`, `package main
+
+func main() {
+}
+`)
+	gopClTestEx(t, &conf, "main", `
+func test() {
+	println "hello"
+}
+`, `package main
+
+import fmt "fmt"
+
+func test() {
+	fmt.Println("hello")
+}
+func main() {
+}
+`)
+
+	gopClTestEx(t, &conf, "main", `
+func main() {
+	println "hello"
+}
+`, `package main
+
+import fmt "fmt"
+
+func main() {
+	fmt.Println("hello")
+}
+`)
+}
