@@ -32,12 +32,12 @@ func codeErrorTest(t *testing.T, msg, src string) {
 
 func codeErrorTestEx(t *testing.T, pkgname, msg, src string) {
 	fs := parsertest.NewSingleFileFS("/foo", "bar.gop", src)
-	pkgs, err := parser.ParseFSDir(gblFset, fs, "/foo", nil, 0)
+	pkgs, err := parser.ParseFSDir(gblFset, fs, "/foo", parser.Config{})
 	if err != nil {
 		scanner.PrintError(os.Stderr, err)
 		t.Fatal("parser.ParseFSDir failed")
 	}
-	conf := *baseConf.Ensure()
+	conf := *gblConf
 	conf.NoFileLine = false
 	conf.WorkingDir = "/foo"
 	conf.TargetDir = "/foo"
@@ -129,6 +129,26 @@ var foo, foo1 func() = nil, => {}
 		"./bar.gop:3:15: lambda unsupport multiple assignment", `
 var foo func()
 _, foo = nil, => {}
+`)
+	codeErrorTest(t,
+		"./bar.gop:4:9: cannot use lambda expression as type int in return statement", `
+func intSeq() int {
+	i := 0
+	return => {
+		i++
+		return i
+	}
+}
+`)
+	codeErrorTest(t,
+		"./bar.gop:6:10: cannot use i (type int) as type string in return argument", `
+func intSeq() func() string {
+	i := 0
+	return => {
+		i++
+		return i
+	}
+}
 `)
 }
 
@@ -806,3 +826,22 @@ import "fmt"
 n, err := fmt.println
 `)
 }
+
+func TestFiledsNameRedecl(t *testing.T) {
+	codeErrorTest(t, `./bar.gop:6:2: Id redeclared
+	./bar.gop:5:2 other declaration of Id
+./bar.gop:7:2: Id redeclared
+	./bar.gop:5:2 other declaration of Id
+./bar.gop:9:2: name redeclared
+	./bar.gop:8:2 other declaration of name`, `
+type Id struct {
+}
+type A struct {
+	Id   int
+	Id   string
+	Id
+	name string
+	name string
+}
+`)
+}
