@@ -29,6 +29,7 @@ import (
 	"github.com/goplus/gop/ast"
 	"github.com/goplus/gop/token"
 	"github.com/goplus/gox"
+	"github.com/goplus/gox/cpackages"
 )
 
 /*-----------------------------------------------------------------------------
@@ -59,9 +60,16 @@ const (
 	clIdentLHS
 	clIdentSelectorExpr
 	clIdentGoto
+	clCallWithTwoValue
 )
 
-func compileIdent(ctx *blockCtx, ident *ast.Ident, flags int) *gox.PkgRef {
+const (
+	objNormal = iota
+	objPkgRef
+	objCPkgRef
+)
+
+func compileIdent(ctx *blockCtx, ident *ast.Ident, flags int) (obj *gox.PkgRef, kind int) {
 	fvalue := (flags&clIdentSelectorExpr) != 0 || (flags&clIdentLHS) == 0
 	name := ident.Name
 	if name == "_" {
@@ -69,7 +77,7 @@ func compileIdent(ctx *blockCtx, ident *ast.Ident, flags int) *gox.PkgRef {
 			panic(ctx.newCodeError(ident.Pos(), "cannot use _ as value"))
 		}
 		ctx.cb.VarRef(nil)
-		return nil
+		return
 	}
 
 	scope := ctx.pkg.Types.Scope()
@@ -80,13 +88,13 @@ func compileIdent(ctx *blockCtx, ident *ast.Ident, flags int) *gox.PkgRef {
 		}
 	}
 
-	if ctx.fileType > 0 { // in a Go+ class file
+	if ctx.isClass { // in a Go+ class file
 		if fn := ctx.cb.Func(); fn != nil {
 			sig := fn.Ancestor().Type().(*types.Signature)
 			if recv := sig.Recv(); recv != nil {
 				ctx.cb.Val(recv)
 				if compileMember(ctx, ident, name, flags) == nil { // class member object
-					return nil
+					return
 				}
 				ctx.cb.InternalStack().PopN(1)
 			}
@@ -103,14 +111,17 @@ func compileIdent(ctx *blockCtx, ident *ast.Ident, flags int) *gox.PkgRef {
 
 	// pkgRef object
 	if (flags & clIdentSelectorExpr) != 0 {
+		if name == "C" && len(ctx.clookups) > 0 {
+			return nil, objCPkgRef
+		}
 		if pr, ok := ctx.findImport(name); ok {
-			return pr
+			return pr, objPkgRef
 		}
 	}
 
 	// object from import . "xxx"
-	if compilePkgRef(ctx, nil, ident, flags) {
-		return nil
+	if compilePkgRef(ctx, nil, ident, flags, objPkgRef) {
+		return
 	}
 
 	// universe object
@@ -133,7 +144,7 @@ find:
 	} else {
 		ctx.cb.VarRef(o, ident)
 	}
-	return nil
+	return
 }
 
 func isBuiltin(o types.Object) bool {
@@ -179,7 +190,11 @@ func compileExpr(ctx *blockCtx, expr ast.Expr, twoValue ...bool) {
 	case *ast.BasicLit:
 		compileBasicLit(ctx, v)
 	case *ast.CallExpr:
-		compileCallExpr(ctx, v, 0)
+		flags := 0
+		if twoValue != nil && twoValue[0] {
+			flags = clCallWithTwoValue
+		}
+		compileCallExpr(ctx, v, flags)
 	case *ast.SelectorExpr:
 		compileSelectorExpr(ctx, v, clIdentAutoCall)
 	case *ast.BinaryExpr:
@@ -191,7 +206,7 @@ func compileExpr(ctx *blockCtx, expr ast.Expr, twoValue ...bool) {
 	case *ast.CompositeLit:
 		compileCompositeLit(ctx, v, nil, false)
 	case *ast.SliceLit:
-		compileSliceLit(ctx, v)
+		compileSliceLit(ctx, v, nil)
 	case *ast.RangeExpr:
 		compileRangeExpr(ctx, v)
 	case *ast.IndexExpr:
@@ -292,7 +307,7 @@ func compileSliceExpr(ctx *blockCtx, v *ast.SliceExpr) { // x[i:j:k]
 func compileSelectorExprLHS(ctx *blockCtx, v *ast.SelectorExpr) {
 	switch x := v.X.(type) {
 	case *ast.Ident:
-		if at := compileIdent(ctx, x, clIdentLHS|clIdentSelectorExpr); at != nil {
+		if at, kind := compileIdent(ctx, x, clIdentLHS|clIdentSelectorExpr); kind != objNormal {
 			ctx.cb.VarRef(at.Ref(v.Sel.Name))
 			return
 		}
@@ -305,8 +320,8 @@ func compileSelectorExprLHS(ctx *blockCtx, v *ast.SelectorExpr) {
 func compileSelectorExpr(ctx *blockCtx, v *ast.SelectorExpr, flags int) {
 	switch x := v.X.(type) {
 	case *ast.Ident:
-		if at := compileIdent(ctx, x, flags|clIdentSelectorExpr); at != nil {
-			if compilePkgRef(ctx, at, v.Sel, flags) {
+		if at, kind := compileIdent(ctx, x, flags|clIdentSelectorExpr); kind != objNormal {
+			if compilePkgRef(ctx, at, v.Sel, flags, kind) {
 				return
 			}
 			if token.IsExported(v.Sel.Name) {
@@ -333,25 +348,39 @@ func pkgRef(at *gox.PkgRef, name string) (o types.Object, alias bool) {
 	return at.TryRef(name), false
 }
 
-func lookupPkgRef(ctx *blockCtx, pkg *gox.PkgRef, x *ast.Ident) (o types.Object, alias bool) {
+func lookupPkgRef(ctx *blockCtx, pkg *gox.PkgRef, x *ast.Ident, pkgKind int) (o types.Object, alias bool) {
 	if pkg != nil {
 		return pkgRef(pkg, x.Name)
 	}
-	for _, at := range ctx.lookups {
-		if o2, alias2 := pkgRef(at, x.Name); o2 != nil {
-			if o != nil {
-				panic(ctx.newCodeErrorf(
-					x.Pos(), "confliction: %s declared both in \"%s\" and \"%s\"",
-					x.Name, at.Types.Path(), pkg.Types.Path()))
+	if pkgKind == objPkgRef {
+		for _, at := range ctx.lookups {
+			if o2, alias2 := pkgRef(at, x.Name); o2 != nil {
+				if o != nil {
+					panic(ctx.newCodeErrorf(
+						x.Pos(), "confliction: %s declared both in \"%s\" and \"%s\"",
+						x.Name, at.Types.Path(), pkg.Types.Path()))
+				}
+				pkg, o, alias = at, o2, alias2
+			}
+		}
+	} else {
+		var cpkg *cpackages.PkgRef
+		for _, at := range ctx.clookups {
+			if o2 := at.Lookup(x.Name); o2 != nil {
+				if o != nil {
+					panic(ctx.newCodeErrorf(
+						x.Pos(), "confliction: %s declared both in \"%s\" and \"%s\"",
+						x.Name, at.Pkg().Types.Path(), cpkg.Pkg().Types.Path()))
+				}
+				cpkg, o = at, o2
 			}
-			pkg, o, alias = at, o2, alias2
 		}
 	}
 	return
 }
 
-func compilePkgRef(ctx *blockCtx, at *gox.PkgRef, x *ast.Ident, flags int) bool {
-	if v, alias := lookupPkgRef(ctx, at, x); v != nil {
+func compilePkgRef(ctx *blockCtx, at *gox.PkgRef, x *ast.Ident, flags, pkgKind int) bool {
+	if v, alias := lookupPkgRef(ctx, at, x, pkgKind); v != nil {
 		cb := ctx.cb
 		if (flags & clIdentLHS) != 0 {
 			cb.VarRef(v, x)
@@ -374,6 +403,7 @@ type fnType struct {
 	params   *types.Tuple
 	n1       int
 	variadic bool
+	typetype bool
 	inited   bool
 }
 
@@ -399,28 +429,43 @@ func (p *fnType) init(t *types.Signature) {
 	}
 }
 
+func (p *fnType) initTypeType(t *gox.TypeType) {
+	param := types.NewParam(0, nil, "", t.Type())
+	p.params, p.typetype = types.NewTuple(param), true
+	p.n1 = 1
+}
+
 func (p *fnType) initWith(fnt types.Type, idx, nin int) {
 	if p.inited {
 		return
 	}
 	p.inited = true
-	if t := gox.CheckSignature(fnt, idx, nin); t != nil {
+	if t, ok := fnt.(*gox.TypeType); ok {
+		p.initTypeType(t)
+	} else if t := gox.CheckSignature(fnt, idx, nin); t != nil {
 		p.init(t)
 	}
 }
 
-func compileCallExpr(ctx *blockCtx, v *ast.CallExpr, flags int) {
+func compileCallExpr(ctx *blockCtx, v *ast.CallExpr, inFlags int) {
 	switch fn := v.Fun.(type) {
 	case *ast.Ident:
-		compileIdent(ctx, fn, clIdentAllowBuiltin|flags)
+		compileIdent(ctx, fn, clIdentAllowBuiltin|inFlags)
 	case *ast.SelectorExpr:
 		compileSelectorExpr(ctx, fn, 0)
 	default:
 		compileExpr(ctx, fn)
 	}
 	var fn fnType
-	fnt := ctx.cb.Get(-1).Type
-	ellipsis := v.Ellipsis != gotoken.NoPos
+	var fnt = ctx.cb.Get(-1).Type
+	var flags gox.InstrFlags
+	var ellipsis = v.Ellipsis != gotoken.NoPos
+	if ellipsis {
+		flags = gox.InstrFlagEllipsis
+	}
+	if (inFlags & clCallWithTwoValue) != 0 {
+		flags |= gox.InstrFlagTwoValue
+	}
 	for i, arg := range v.Args {
 		switch expr := arg.(type) {
 		case *ast.LambdaExpr:
@@ -434,11 +479,31 @@ func compileCallExpr(ctx *blockCtx, v *ast.CallExpr, flags int) {
 		case *ast.CompositeLit:
 			fn.initWith(fnt, i, -1)
 			compileCompositeLit(ctx, expr, fn.arg(i, ellipsis), true)
+		case *ast.SliceLit:
+			fn.initWith(fnt, i, -2)
+			t := fn.arg(i, ellipsis)
+			switch t.(type) {
+			case *types.Slice:
+			case *types.Named:
+				if _, ok := getUnderlying(ctx, t).(*types.Slice); !ok {
+					t = nil
+				}
+			default:
+				t = nil
+			}
+			typetype := fn.typetype && t != nil && len(v.Args) == 1
+			if typetype {
+				ctx.cb.InternalStack().Pop()
+			}
+			compileSliceLit(ctx, expr, t)
+			if typetype {
+				return
+			}
 		default:
 			compileExpr(ctx, arg)
 		}
 	}
-	ctx.cb.CallWith(len(v.Args), ellipsis, v)
+	ctx.cb.CallWith(len(v.Args), flags, v)
 }
 
 type clLambaFlag string
@@ -479,7 +544,7 @@ func compileLambda(ctx *blockCtx, lambda ast.Expr, sig *types.Signature) {
 	}
 }
 
-func compileLambdaParams(ctx *blockCtx, pos token.Pos, lhs []*ast.Ident, in *types.Tuple) []*types.Var {
+func makeLambdaParams(ctx *blockCtx, pos token.Pos, lhs []*ast.Ident, in *types.Tuple) *types.Tuple {
 	pkg := ctx.pkg
 	n := len(lhs)
 	if nin := in.Len(); n != nin {
@@ -494,64 +559,83 @@ func compileLambdaParams(ctx *blockCtx, pos token.Pos, lhs []*ast.Ident, in *typ
 		panic(ctx.newCodeErrorf(
 			pos, "too %s arguments in lambda expression\n\thave (%s)\n\twant %v", fewOrMany, strings.Join(has, ", "), in))
 	}
+	if n == 0 {
+		return nil
+	}
 	params := make([]*types.Var, n)
 	for i, name := range lhs {
 		params[i] = pkg.NewParam(name.Pos(), name.Name, in.At(i).Type())
 	}
-	return params
+	return types.NewTuple(params...)
+}
+
+func makeLambdaResults(pkg *gox.Package, out *types.Tuple) *types.Tuple {
+	nout := out.Len()
+	if nout == 0 {
+		return nil
+	}
+	results := make([]*types.Var, nout)
+	for i := 0; i < nout; i++ {
+		results[i] = pkg.NewParam(token.NoPos, "", out.At(i).Type())
+	}
+	return types.NewTuple(results...)
 }
 
 func compileLambdaExpr(ctx *blockCtx, v *ast.LambdaExpr, sig *types.Signature) {
 	pkg := ctx.pkg
-	params := compileLambdaParams(ctx, v.Pos(), v.Lhs, sig.Params())
-	out := sig.Results()
-	nout := out.Len()
-	results := make([]*types.Var, nout)
-	for i := 0; i < nout; i++ {
-		results[i] = pkg.NewParam(token.NoPos, "", out.At(i).Type())
-	}
-	ctx.cb.NewClosure(types.NewTuple(params...), types.NewTuple(results...), false).BodyStart(pkg)
+	params := makeLambdaParams(ctx, v.Pos(), v.Lhs, sig.Params())
+	results := makeLambdaResults(pkg, sig.Results())
+	ctx.cb.NewClosure(params, results, false).BodyStart(pkg)
 	for _, v := range v.Rhs {
 		compileExpr(ctx, v)
 	}
-	ctx.cb.Return(nout).End()
+	ctx.cb.Return(len(v.Rhs)).End()
 }
 
 func compileLambdaExpr2(ctx *blockCtx, v *ast.LambdaExpr2, sig *types.Signature) {
 	pkg := ctx.pkg
-	params := compileLambdaParams(ctx, v.Pos(), v.Lhs, sig.Params())
-	cb := ctx.cb
-	comments := cb.Comments()
-	out := sig.Results()
-	nout := out.Len()
-	results := make([]*types.Var, nout)
-	for i := 0; i < nout; i++ {
-		results[i] = pkg.NewParam(token.NoPos, "", out.At(i).Type())
-	}
-	fn := cb.NewClosure(types.NewTuple(params...), types.NewTuple(results...), false)
+	params := makeLambdaParams(ctx, v.Pos(), v.Lhs, sig.Params())
+	results := makeLambdaResults(pkg, sig.Results())
+	comments, once := ctx.cb.BackupComments()
+	fn := ctx.cb.NewClosure(params, results, false)
 	loadFuncBody(ctx, fn, v.Body)
-	cb.SetComments(comments, false)
+	ctx.cb.SetComments(comments, once)
 }
 
 func compileFuncLit(ctx *blockCtx, v *ast.FuncLit) {
 	cb := ctx.cb
-	comments := cb.Comments()
+	comments, once := cb.BackupComments()
 	sig := toFuncType(ctx, v.Type, nil)
 	fn := cb.NewClosureWith(sig)
 	if body := v.Body; body != nil {
 		loadFuncBody(ctx, fn, body)
-		cb.SetComments(comments, false)
+		cb.SetComments(comments, once)
 	}
 }
 
 func compileBasicLit(ctx *blockCtx, v *ast.BasicLit) {
-	if v.Kind == token.RAT {
+	cb := ctx.cb
+	switch v.Kind {
+	case token.RAT:
 		val := v.Value
 		bi, _ := new(big.Int).SetString(val[:len(val)-1], 10) // remove r suffix
-		ctx.cb.UntypedBigInt(bi, v)
-		return
+		cb.UntypedBigInt(bi, v)
+	case token.CSTRING:
+		s, err := strconv.Unquote(v.Value)
+		if err != nil {
+			log.Panicln("compileBasicLit:", err)
+		}
+		n := len(s)
+		tyInt8 := types.Typ[types.Int8]
+		typ := types.NewArray(tyInt8, int64(n+1))
+		cb.Typ(types.NewPointer(tyInt8)).Typ(types.Typ[types.UnsafePointer])
+		for i := 0; i < n; i++ {
+			cb.Val(rune(s[i]))
+		}
+		cb.Val(rune(0)).ArrayLit(typ, n+1).UnaryOp(gotoken.AND).Call(1).Call(1)
+	default:
+		cb.Val(&goast.BasicLit{Kind: gotoken.Token(v.Kind), Value: v.Value}, v)
 	}
-	ctx.cb.Val(&goast.BasicLit{Kind: gotoken.Token(v.Kind), Value: v.Value}, v)
 }
 
 const (
@@ -597,12 +681,14 @@ func compileCompositeLitElts(ctx *blockCtx, elts []ast.Expr, kind int, expected
 func compileStructLitInKeyVal(ctx *blockCtx, elts []ast.Expr, t *types.Struct, typ types.Type) {
 	for _, elt := range elts {
 		kv := elt.(*ast.KeyValueExpr)
-		name := kv.Key.(*ast.Ident).Name
-		idx := lookupField(t, name)
+		name := kv.Key.(*ast.Ident)
+		idx := lookupField(t, name.Name)
 		if idx >= 0 {
 			ctx.cb.Val(idx)
 		} else {
-			log.Panicln("TODO: struct member not found -", name)
+			src, pos := ctx.LoadExpr(name)
+			err := newCodeErrorf(&pos, "%s undefined (type %v has no field or method %s)", src, typ, name.Name)
+			panic(err)
 		}
 		switch expr := kv.Value.(type) {
 		case *ast.LambdaExpr, *ast.LambdaExpr2:
@@ -719,12 +805,12 @@ func compileCompositeLit(ctx *blockCtx, v *ast.CompositeLit, expected types.Type
 	}
 }
 
-func compileSliceLit(ctx *blockCtx, v *ast.SliceLit) {
+func compileSliceLit(ctx *blockCtx, v *ast.SliceLit, typ types.Type) {
 	n := len(v.Elts)
 	for _, elt := range v.Elts {
 		compileExpr(ctx, elt)
 	}
-	ctx.cb.SliceLit(nil, n)
+	ctx.cb.SliceLit(typ, n)
 }
 
 func compileRangeExpr(ctx *blockCtx, v *ast.RangeExpr) {
