@@ -164,7 +164,18 @@ func compileReturnStmt(ctx *blockCtx, expr *ast.ReturnStmt) {
 					twoValue = (results.Len() == 2)
 				}
 			}
-			compileExpr(ctx, ret, twoValue)
+			switch v := ret.(type) {
+			case *ast.LambdaExpr, *ast.LambdaExpr2:
+				rtyp := ctx.cb.Func().Type().(*types.Signature).Results().At(i).Type()
+				sig, ok := rtyp.(*types.Signature)
+				if !ok {
+					panic(ctx.newCodeErrorf(
+						ret.Pos(), "cannot use lambda expression as type %v in return statement", rtyp))
+				}
+				compileLambda(ctx, v, sig)
+			default:
+				compileExpr(ctx, ret, twoValue)
+			}
 		}
 	}
 	ctx.cb.Return(len(expr.Results), expr)
@@ -247,11 +258,11 @@ func compileRangeStmt(ctx *blockCtx, v *ast.RangeStmt) {
 		if v.Tok == token.ASSIGN {
 			tok = v.Tok
 		}
-		compileForStmt(ctx, toForStmt(v.For, v.Key, v.Body, re, tok))
+		compileForStmt(ctx, toForStmt(v.For, v.Key, v.Body, re, tok, nil))
 		return
 	}
 	cb := ctx.cb
-	comments := cb.Comments()
+	comments, once := cb.BackupComments()
 	if v.Tok == token.DEFINE {
 		names := make([]string, 1, 2)
 		if v.Key == nil {
@@ -288,18 +299,18 @@ func compileRangeStmt(ctx *blockCtx, v *ast.RangeStmt) {
 	}
 	cb.RangeAssignThen(pos)
 	compileStmts(ctx, v.Body.List)
-	cb.SetComments(comments, true)
+	cb.SetComments(comments, once)
 	setBodyHandler(ctx)
 	cb.End()
 }
 
 func compileForPhraseStmt(ctx *blockCtx, v *ast.ForPhraseStmt) {
 	if re, ok := v.X.(*ast.RangeExpr); ok {
-		compileForStmt(ctx, toForStmt(v.For, v.Value, v.Body, re, token.DEFINE))
+		compileForStmt(ctx, toForStmt(v.For, v.Value, v.Body, re, token.DEFINE, v.ForPhrase))
 		return
 	}
 	cb := ctx.cb
-	comments := cb.Comments()
+	comments, once := cb.BackupComments()
 	names := make([]string, 1, 2)
 	if v.Key == nil {
 		names[0] = "_"
@@ -317,17 +328,17 @@ func compileForPhraseStmt(ctx *blockCtx, v *ast.ForPhraseStmt) {
 		compileExpr(ctx, v.Cond)
 		cb.Then()
 		compileStmts(ctx, v.Body.List)
-		cb.SetComments(comments, true)
+		cb.SetComments(comments, once)
 		cb.End()
 	} else {
 		compileStmts(ctx, v.Body.List)
+		cb.SetComments(comments, once)
 	}
-	cb.SetComments(comments, true)
 	setBodyHandler(ctx)
 	cb.End()
 }
 
-func toForStmt(forPos token.Pos, value ast.Expr, body *ast.BlockStmt, re *ast.RangeExpr, tok token.Token) *ast.ForStmt {
+func toForStmt(forPos token.Pos, value ast.Expr, body *ast.BlockStmt, re *ast.RangeExpr, tok token.Token, fp *ast.ForPhrase) *ast.ForStmt {
 	nilIdent := value == nil
 	if !nilIdent {
 		if v, ok := value.(*ast.Ident); ok && v.Name == "_" {
@@ -337,11 +348,12 @@ func toForStmt(forPos token.Pos, value ast.Expr, body *ast.BlockStmt, re *ast.Ra
 	if nilIdent {
 		value = &ast.Ident{NamePos: forPos, Name: "_gop_k"}
 	}
-	if re.First == nil {
-		re.First = &ast.BasicLit{ValuePos: forPos, Kind: token.INT, Value: "0"}
+	first := re.First
+	if first == nil {
+		first = &ast.BasicLit{ValuePos: forPos, Kind: token.INT, Value: "0"}
 	}
 	initLhs := []ast.Expr{value}
-	initRhs := []ast.Expr{re.First}
+	initRhs := []ast.Expr{first}
 	replaceValue := false
 	var cond ast.Expr
 	var post ast.Expr
@@ -379,6 +391,16 @@ func toForStmt(forPos token.Pos, value ast.Expr, body *ast.BlockStmt, re *ast.Ra
 		}}, body.List...)
 		tok = token.DEFINE
 	}
+	if fp != nil && fp.Cond != nil {
+		condStmt := &ast.IfStmt{
+			Init: fp.Init,
+			Cond: fp.Cond,
+			Body: body,
+		}
+		body = &ast.BlockStmt{
+			List: []ast.Stmt{condStmt},
+		}
+	}
 	return &ast.ForStmt{
 		For: forPos,
 		Init: &ast.AssignStmt{
@@ -409,7 +431,7 @@ func toForStmt(forPos token.Pos, value ast.Expr, body *ast.BlockStmt, re *ast.Ra
 // end
 func compileForStmt(ctx *blockCtx, v *ast.ForStmt) {
 	cb := ctx.cb
-	comments := cb.Comments()
+	comments, once := cb.BackupComments()
 	cb.For()
 	if v.Init != nil {
 		compileStmt(ctx, v.Init)
@@ -425,7 +447,7 @@ func compileForStmt(ctx *blockCtx, v *ast.ForStmt) {
 		cb.Post()
 		compileStmt(ctx, v.Post)
 	}
-	cb.SetComments(comments, true)
+	cb.SetComments(comments, once)
 	setBodyHandler(ctx)
 	cb.End()
 }
@@ -435,7 +457,7 @@ func compileForStmt(ctx *blockCtx, v *ast.ForStmt) {
 // end
 func compileIfStmt(ctx *blockCtx, v *ast.IfStmt) {
 	cb := ctx.cb
-	comments := cb.Comments()
+	comments, once := cb.BackupComments()
 	cb.If()
 	if v.Init != nil {
 		compileStmt(ctx, v.Init)
@@ -451,7 +473,7 @@ func compileIfStmt(ctx *blockCtx, v *ast.IfStmt) {
 			compileStmt(ctx, e)
 		}
 	}
-	cb.SetComments(comments, true)
+	cb.SetComments(comments, once)
 	cb.End()
 }
 
@@ -465,7 +487,7 @@ func compileIfStmt(ctx *blockCtx, v *ast.IfStmt) {
 // end
 func compileTypeSwitchStmt(ctx *blockCtx, v *ast.TypeSwitchStmt) {
 	var cb = ctx.cb
-	comments := cb.Comments()
+	comments, once := cb.BackupComments()
 	var name string
 	var ta *ast.TypeAssertExpr
 	switch stmt := v.Assign.(type) {
@@ -529,7 +551,7 @@ func compileTypeSwitchStmt(ctx *blockCtx, v *ast.TypeSwitchStmt) {
 		commentStmt(ctx, stmt)
 		cb.End()
 	}
-	cb.SetComments(comments, true)
+	cb.SetComments(comments, once)
 	cb.End()
 }
 
@@ -543,7 +565,7 @@ func compileTypeSwitchStmt(ctx *blockCtx, v *ast.TypeSwitchStmt) {
 // end
 func compileSwitchStmt(ctx *blockCtx, v *ast.SwitchStmt) {
 	cb := ctx.cb
-	comments := cb.Comments()
+	comments, once := cb.BackupComments()
 	cb.Switch()
 	if v.Init != nil {
 		compileStmt(ctx, v.Init)
@@ -604,7 +626,7 @@ func compileSwitchStmt(ctx *blockCtx, v *ast.SwitchStmt) {
 		commentStmt(ctx, stmt)
 		cb.End()
 	}
-	cb.SetComments(comments, true)
+	cb.SetComments(comments, once)
 	cb.End()
 }
 
@@ -631,7 +653,7 @@ func hasFallthrough(body []ast.Stmt) ([]ast.Stmt, bool) {
 // end
 func compileSelectStmt(ctx *blockCtx, v *ast.SelectStmt) {
 	cb := ctx.cb
-	comments := cb.Comments()
+	comments, once := cb.BackupComments()
 	cb.Select()
 	for _, stmt := range v.Body.List {
 		c, ok := stmt.(*ast.CommClause)
@@ -648,7 +670,7 @@ func compileSelectStmt(ctx *blockCtx, v *ast.SelectStmt) {
 		commentStmt(ctx, stmt)
 		cb.End()
 	}
-	cb.SetComments(comments, true)
+	cb.SetComments(comments, once)
 	cb.End()
 }
 
@@ -728,6 +750,9 @@ func compileDeclStmt(ctx *blockCtx, expr *ast.DeclStmt) {
 
 func compileType(ctx *blockCtx, t *ast.TypeSpec) {
 	name := t.Name.Name
+	if name == "_" {
+		return
+	}
 	if t.Assign != token.NoPos { // alias type
 		ctx.cb.AliasType(name, toType(ctx, t.Type))
 	} else {
