@@ -13,17 +13,23 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-// gop mod tidy
+
 package mod
 
 import (
+	"fmt"
+	"log"
+	"os"
+	"syscall"
+
+	"github.com/goplus/gop"
 	"github.com/goplus/gop/cmd/internal/base"
-	"github.com/goplus/gop/cmd/internal/modload"
-	"github.com/goplus/gop/x/mod/modfetch"
+	"github.com/goplus/gop/x/gopenv"
 )
 
+// gop mod tidy
 var cmdTidy = &base.Command{
-	UsageLine: "gop mod tidy [-e] [-v]",
+	UsageLine: "gop mod tidy [-e -v]",
 	Short:     "add missing and remove unused modules",
 }
 
@@ -32,8 +38,12 @@ func init() {
 }
 
 func runTidy(cmd *base.Command, args []string) {
-	modload.LoadModFile()
-	modload.SyncGoMod()
-	modfetch.TidyArgs(".", args...)
-	modload.SyncGopMod()
+	err := gop.Tidy(".", gopenv.Get())
+	if err != nil {
+		if err == syscall.ENOENT {
+			fmt.Fprintln(os.Stderr, "gop.mod not found")
+			os.Exit(1)
+		}
+		log.Panicln(err)
+	}
 }
